Struct own_ref::Slot

source ·
pub struct Slot<T>(/* private fields */);
The output of slot().

Implementations

source

impl<T> Slot<T>

source

pub const VACANT: Self = _

Same as slot(). Or rather, slot() is the same as Slot::VACANT.

The issue with the clearly zero-cost Slot::VACANT expression is that Rust may lint against &mut <constant>, such as with let slot = &mut Slot::VACANT;, which it does not do for let slot = &mut slot().

source

pub fn holding<'slot>(self: &'slot mut Slot<T>, value: T) -> OwnRef<'slot, T>

Main non-macro, non-scoped, non-unsafe constructor for an OwnRef.